/*
 * JScience - Java(TM) Tools and Libraries for the Advancement of Sciences.
 * Copyright (C) 2005 - JScience (http://jscience.org/)
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software is
 * freely granted, provided that this notice is preserved.
 */
package org.jscience.mathematics.functions;

import java.util.SortedMap;
import org.jscience.mathematics.matrices.Operable;

/**
 * <p>
 * This class represents an estimator of the values at a certain point using
 * surrounding points and values.
 * </p>
 * 
 * @author <a href="mailto:jean-marie@dautelle.com">Jean-Marie Dautelle </a>
 * @version 1.0, January 24, 2004
 */
public abstract class Interpolator {

    /**
     * Default constructor.
     */
    protected Interpolator() {
    }

    /**
     * Holds a simple linear interpolator upon
     * {@link org.jscience.mathematics.matrices.Operable Operable}points and
     * values.
     */
    public final static Interpolator LINEAR = new Interpolator() {

        // Implements Interpolator.
        public Operable interpolate(Comparable point, SortedMap pointValues) {

            // Searches exact.
            Operable y = (Operable) pointValues.get(point);
            if (y != null)
                return y;

            // Searches surrounding points/values.
            SortedMap headMap = pointValues.headMap(point);
            Operable x1 = (Operable) headMap.lastKey();
            Operable y1 = (Operable) headMap.get(x1);
            SortedMap tailMap = pointValues.tailMap(point);
            Operable x2 = (Operable) tailMap.lastKey();
            Operable y2 = (Operable) tailMap.get(x2);

            // Interpolates.
            final Operable x = (Operable) point; // Requires Operable value.
            Operable deltaInv = x2.plus(x1.opposite()).reciprocal();
            Operable k1 = x2.plus(x.opposite()).times(deltaInv);
            Operable k2 = x.plus(x1.opposite()).times(deltaInv);
            return y1.times(k1).plus(y2.times(k2));
        }
    };

    /**
     * Estimates the value at the specified point.
     * 
     * @param point
     *            the point for which the value is estimated.
     * @param pointValues
     *            the point-value entries.
     * @return the estimated value at the specified point.
     */
    public abstract Operable interpolate(Comparable point, SortedMap pointValues);
}