/*
 * JScience - Java(TM) Tools and Libraries for the Advancement of Sciences.
 * Copyright (C) 2005 - JScience (http://jscience.org/)
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software is
 * freely granted, provided that this notice is preserved.
 */
package org.jscience.physics.quantities;
import org.jscience.physics.units.ConversionException;
import org.jscience.physics.units.SI;
import org.jscience.physics.units.Unit;

/**
 * This class represents a catalytic activity. The system unit for this quantity
 * is "mol/s". By default, instances of this class are showed in Katal ("kat").
 *
 * @author  <a href="mailto:jean-marie@dautelle.com">Jean-Marie Dautelle</a>
 * @version 1.0, October 24, 2004
 */
public class CatalyticActivity extends Quantity {

    /**
     * Holds the system unit.
     */
    private final static Unit SYSTEM_UNIT = SI.KATAL.getDimension();

    /**
     * Holds the factory for this class.
     */
    private final static Factory FACTORY = new Factory(SYSTEM_UNIT) {
        protected Quantity newQuantity() {
            return new CatalyticActivity();
        }
    }.useFor(SI.KATAL);

    /**
     * Represents a {@link CatalyticActivity} amounting to nothing.
     */
    public final static CatalyticActivity ZERO = (CatalyticActivity) valueOf(0,
            SYSTEM_UNIT);

    /**
     * Default constructor (allows for derivation).
     */
    protected CatalyticActivity() {
    }

    /**
     * Returns the {@link CatalyticActivity} corresponding to the specified
     * quantity.
     *
     * @param  quantity a quantity compatible with {@link CatalyticActivity}.
     * @return the specified quantity or a new {@link CatalyticActivity} instance.
     * @throws ConversionException if the current model does not allow the
     *         specified quantity to be converted to {@link CatalyticActivity}.
     */
    public static CatalyticActivity CatalyticActivityOf(Quantity quantity) {
        return (CatalyticActivity) FACTORY.quantity(quantity);
    }

    /**
     * Shows {@link CatalyticActivity} instances in the specified unit.
     *
     * @param  unit the output unit for {@link CatalyticActivity} instances.
     * @see    Quantity#getOutputUnit
     */
    public static void showAs(Unit unit) {
        FACTORY.showInstancesAs(unit);
    }

    private static final long serialVersionUID = -6532456610018465419L;
}