/*
 * JScience - Java(TM) Tools and Libraries for the Advancement of Sciences.
 * Copyright (C) 2005 - JScience (http://jscience.org/)
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software is
 * freely granted, provided that this notice is preserved.
 */
package org.jscience.physics.quantities;
import org.jscience.physics.units.ConversionException;
import org.jscience.physics.units.SI;
import org.jscience.physics.units.Unit;

/**
 * This class represents the speed of data-transmission.
 * The system unit for this quantity is "bit/s".
 *
 * @author  <a href="mailto:jean-marie@dautelle.com">Jean-Marie Dautelle</a>
 * @version 1.0, October 24, 2004
 */
public class DataRate extends Quantity {

    /**
     * Holds the system unit.
     */
    private final static Unit SYSTEM_UNIT = SI.BIT.divide(SI.SECOND);

    /**
     * Holds the factory for this class.
     */
    private final static Factory FACTORY = new Factory(SYSTEM_UNIT) {
        protected Quantity newQuantity() {
             return new DataRate();
        }
    };

    /**
     * Represents a {@link DataRate} amounting to nothing.
     */
    public final static DataRate ZERO = (DataRate) valueOf(0, SYSTEM_UNIT);

    /**
     * Default constructor (allows for derivation).
     */
    protected DataRate() {}

    /**
     * Returns the {@link DataRate} corresponding to the specified quantity.
     *
     * @param  quantity a quantity compatible with {@link DataRate}.
     * @return the specified quantity or a new {@link DataRate} instance.
     * @throws ConversionException if the current model does not allow the
     *         specified quantity to be converted to {@link DataRate}.
     */
    public static DataRate dataRateOf(Quantity quantity) {
        return (DataRate) FACTORY.quantity(quantity);
    }

    /**
     * Shows {@link DataRate} instances in the specified unit.
     *
     * @param  unit the output unit for {@link DataRate} instances.
     * @see    Quantity#getOutputUnit
     */
    public static void showAs(Unit unit) {
        FACTORY.showInstancesAs(unit);
    }

    private static final long serialVersionUID = -516614320976377233L;
}
